#!/usr/bin/env python

import sys
from qt import *

TRUE  = 1
FALSE = 0
ourDrawFunction = ['drawColorWhel','drawFonts','drawShapes']

def drawColorWhel(p):
    f = QFont('times',18,QFont.Bold)
    p.setFont(f)
    p.setPen(Qt.black)
    p.setWindow(0,0,500,500)
    for i in range(36):
        matrix = QWMatrix()
        matrix.translate(250.0,250.0)
        matrix.shear(0.0,0.3)
        matrix.rotate(i*10.0)
        p.setWorldMatrix(matrix)

        c = QColor()
        c.setHsv(i*10,255,255)
        p.setBrush(c)
        p.drawRect(70,-10,80,10)

        n = 'H=' + str(i*10)
        p.drawText(80+70+5,0,n)

def drawFonts(p):
    fonts = ['Helvetica','Courier','Times',0]
    size = [10,12,18,24,36,0]
    f = 0
    y = 0
    while fonts[f]:
        s = 0
        while size[s]:
            font = QFont(fonts[f],size[s])
            p.setFont(font)
            fm = p.fontMetrics()
            y = y + fm.ascent()
            p.drawText(10,y,'Quartz Glyph Job Vex\'d Cwm Finks')
            y = y + fm.descent()
            s = s + 1
        f = f + 1

def drawShapes(p):
    b1 = QBrush(Qt.blue)
    b2 = QBrush(Qt.green,Qt.Dense6Pattern)
    b3 = QBrush(Qt.NoBrush)
    b4 = QBrush(Qt.CrossPattern)

    p.setPen(Qt.red)
    p.setBrush(b1)
    p.drawRect(10,10,200,100)
    p.setBrush(b2)
    p.drawRoundRect(10,150,200,100,20,20)
    p.setBrush(b3)
    p.drawEllipse(250,10,200,100)
    p.setBrush(b4)
    p.drawPie(250,150,200,100,45*16,90*16)

class DrawView(QWidget):
    def __init__(self):
        QWidget.__init__(self)
        self.setCaption('PyQt Draw Demo Application')

        self.bgroup = QButtonGroup(self)
        self.bgroup.resize(200,200)
        self.connect(self.bgroup,SIGNAL('clicked(int)'),self.updateIt)
        self.bgroup.setPaletteBackgroundColor(self.colorGroup().background())
        self.setPaletteBackgroundColor(Qt.white)

        maxwidth = 80
        maxheight = 10

        fm = self.bgroup.fontMetrics()

        for n in ourDrawFunction:
            w = fm.width(n)
            if w > maxwidth:
                maxwidth = w

        maxwidth = maxwidth + 30

        i = 0
        for n in ourDrawFunction:
            rb = QRadioButton(n,self.bgroup)
            rb.setGeometry(10,i*30+10,maxwidth,30)
            maxheight = maxheight + 30
            if i == 0:
                rb.setChecked(TRUE)
            i = i + 1
        maxheight = maxheight + 10
        self.drawindex = 0
        self.maxindex = i
        maxwidth = maxwidth + 20
        self.bgroup.resize(maxwidth,maxheight)

        self.printer = QPrinter()
        self._print = QPushButton('Print...',self.bgroup)
        self._print.resize(80,30)
        self._print.move(maxwidth/2-self._print.width()/2,self.maxindex*30+20)
        self.connect(self._print,SIGNAL('clicked()'),self.printIt)

        self.bgroup.resize(maxwidth,self._print.y()+self._print.height()+10)

        self.resize(640,300)

    def updateIt(self,i):
        if i < self.maxindex:
            self.drawindex = i
            self.update()

    def drawIt(self,p):
        if self.drawindex == 0:
            drawColorWhel(p)
        elif self.drawindex == 1:
            drawFonts(p)
        elif self.drawindex == 2:
            drawShapes(p)

    def printIt(self):
        if self.printer.setup(self):
            paint = QPainter(self.printer)
            self.drawIt(paint)

    def paintEvent(self,e):
        paint = QPainter(self)
        self.drawIt(paint)

    def resizeEvent(self,e):
        self.bgroup.move(self.width()-self.bgroup.width(),0)

## main program
a = QApplication(sys.argv)

draw = DrawView()
draw.setCaption('PyQT Example - Drawdemo')
a.setMainWidget(draw)
draw.show()

a.exec_loop()