#!/usr/bin/env python

import sys
from qt import *

TRUE  = 1
FALSE = 0

class Viewer(QWidget):
    def __init__(self,parent=None,name=None):
        QWidget.__init__(self,parent,name)
	self.setFontSubstitutions()
	greeting_heb = unicode('\327\251\327\234\327\225\327\235','UTF-8')
	greeting_ru = unicode('\320\227\320\264\321\200\320\260\320\262\321\201\321\202\320\262\321\203\320\271\321\202\320\265','UTF-8')
	greeting_en = unicode('Hello','UTF-8')

	self.greetings = QTextView(self,'textview')
	self.greetings.setText(greeting_en + '\n' + greeting_ru + '\n' + greeting_heb)

	self.fontInfo = QTextView(self,'fontinfo')

	self.setDefault()

	self.defaultButton = QPushButton('Default',self,'pushbutton1')
	self.defaultButton.setFont(QFont('times'))
	self.connect(self.defaultButton,SIGNAL('clicked()'),self.setDefault)
	
	self.sansSerifButton = QPushButton('Sans Serif',self,'pushbutton2')
	self.sansSerifButton.setFont(QFont('Hevletika',12))
	self.connect(self.sansSerifButton,SIGNAL('clicked()'),self.setSansSerif)

	self.italicsButton = QPushButton('Italics',self,'pushbutton3')
	self.italicsButton.setFont(QFont('Italics',12,QFont.Bold,TRUE))
	self.connect(self.italicsButton,SIGNAL('clicked()'),self.setItalics)

	self.layout()

    def setFontSubstitutions(self):
	substitutes1 = QStringList()
	substitutes1.append('Times')
	substitutes1.append('Mincho')
	substitutes1.append('Arabic Newspaper')
	substitutes1.append('crox')

	QFont.insertSubstitutions('Bavaria',substitutes1)

	substitutes2 = QStringList()
	substitutes2.append('Lucida')

	QFont.insertSubstitutions('Tokyo',substitutes2)

    def setDefault(self):
	self.font = QFont('Bavaria')
	self.font.setPointSize(24)

	self.font.setWeight(QFont.Bold)
	self.font.setUnderline(TRUE)

	self.greetings.setFont(self.font)

	self.showFontInfo(self.font)

    def setSansSerif(self):
	font = QFont('Newyork',18)
	font.setStyleHint(QFont.SansSerif)

	self.greetings.setFont(font)

	self.showFontInfo(font)

    def setItalics(self):
	font = QFont('Tokyo')
	font.setPointSize(32)
	font.setWeight(QFont.Bold)
	font.setItalic(TRUE)

	self.greetings.setFont(font)

	self.showFontInfo(font)

    def showFontInfo(self,font):
	info = QFontInfo(font)

	messageText = 'Font requested: \'' + str(font.family()) + '\' ' + str(font.pointSize()) + 'pt<BR>' + 'Font used: \'' + str(info.family()) + '\' ' + str(info.pointSize()) + 'pt<P>'
	substitutions = QFont.substitutes(font.family())

	if not substitutions.isEmpty():
	   messageText = messageText + 'The following substitutions exist for ' + str(font.family()) + ':<UL>'
	   
	   messageText = messageText + str(substitutions.join(', '))
	else:
	   messageText = messageText + 'No substitutions exist for ' + str(font.family()) + '.'

	self.fontInfo.setText(messageText)


    def layout(self):
	textViewContainer = QHBoxLayout()

	textViewContainer.addWidget(self.greetings)
	textViewContainer.addWidget(self.fontInfo)

	buttonContainer = QHBoxLayout()

	buttonContainer.addWidget(self.defaultButton)
	buttonContainer.addWidget(self.sansSerifButton)
	buttonContainer.addWidget(self.italicsButton)

	maxButtonHeight = self.defaultButton.height()

	self.defaultButton.setFixedHeight(maxButtonHeight)
	self.sansSerifButton.setFixedHeight(maxButtonHeight)
	self.italicsButton.setFixedHeight(maxButtonHeight)

	container = QVBoxLayout(self)
	container.addLayout(textViewContainer)
	container.addLayout(buttonContainer)

	self.resize(700,250)

## main program
a = QApplication(sys.argv)
textViewer = Viewer()
a.setMainWidget(textViewer)
textViewer.setCaption('PyQt Example - Simple QFont Demo')
textViewer.show()

a.exec_loop()
