#!/usr/bin/env python

import sys, string
from qt import *
from os import *

TRUE  = 1
FALSE = 0

fileopen = [
'    16    13        5            1',
'. c #040404',
'# c #808304',
'a c None',
'b c #f3f704',
'c c #f3f7f3',
'aaaaaaaaa...aaaa',
'aaaaaaaa.aaa.a.a',
'aaaaaaaaaaaaa..a',
'a...aaaaaaaa...a',
'.bcb.......aaaaa',
'.cbcbcbcbc.aaaaa',
'.bcbcbcbcb.aaaaa',
'.cbcb...........',
'.bcb.#########.a',
'.cb.#########.aa',
'.b.#########.aaa',
'..#########.aaaa',
'...........aaaaa'
]

filenew = [
'16 16 3 1',
'       c None',
'.      c #000000000000',
'X      c #FFFFFFFFFFFF',
'                ',
'   ......       ',
'   .XXX.X.      ',
'   .XXX.XX.     ',
'   .XXX.XXX.    ',
'   .XXX.....    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .XXXXXXX.    ',
'   .........    ',
'                ',
'                '
]

back = [
'16 16 5 1',
'# c #000000',
'a c #ffffff',
'c c #808080',
'b c #c0c0c0',
'. c None',
'................',
'.......#........',
'......##........',
'.....#a#........',
'....#aa########.',
'...#aabaaaaaaa#.',
'..#aabbbbbbbbb#.',
'...#abbbbbbbbb#.',
'...c#ab########.',
'....c#a#ccccccc.',
'.....c##c.......',
'......c#c.......',
'.......cc.......',
'........c.......',
'................',
'......................']

forward = [
'16 16 5 1',
'# c #000000',
'a c #ffffff',
'c c #808080',
'b c #c0c0c0',
'. c None',
'................',
'................',
'.........#......',
'.........##.....',
'.........#a#....',
'..########aa#...',
'..#aaaaaaabaa#..',
'..#bbbbbbbbbaa#.',
'..#bbbbbbbbba#..',
'..########ba#c..',
'..ccccccc#a#c...',
'........c##c....',
'........c#c.....',
'........cc......',
'........c.......',
'................',
'................']

homes = [
'16 16 4 1',
'# c #000000',
'a c #ffffff',
'b c #c0c0c0',
'. c None',
'........... ....',
'   ....##.......',
'..#...####......',
'..#..#aabb#.....',
'..#.#aaaabb#....',
'..##aaaaaabb#...',
'..#aaaaaaaabb#..',
'.#aaaaaaaaabbb#.',
'###aaaaaaaabb###',
'..#aaaaaaaabb#..',
'..#aaa###aabb#..',
'..#aaa#.#aabb#..',
'..#aaa#.#aabb#..',
'..#aaa#.#aabb#..',
'..#aaa#.#aabb#..',
'..#####.######..',
'................']

class HelpWindow(QMainWindow):
    def __init__(self,home_,_path,parent=None,name=None):
        QMainWindow.__init__(self,parent,name,Qt.WDestructiveClose)
        self.pathCombo = 0
        self.history = []
        self.bookmarks = []
        self.mHistory = {}
        self.mBookmarks = {}
        self.readHistory()
        self.readBookmarks()

        self.browser = QTextBrowser(self)

        self.browser.mimeSourceFactory().setFilePath(_path)
        self.browser.setFrameStyle(QFrame.Panel|QFrame.Sunken)
        self.connect(self.browser,SIGNAL('sourceChanged(const QString&)'),self.slotSourceChanged)

        self.setCentralWidget(self.browser)

        if home_ != '':
            self.browser.setSource(home_)

        self.connect(self.browser,SIGNAL('highlighted(const QString&)'),self.statusBar(),SLOT('message(const QString&)'))

        self.resize(640,700)

        file = QPopupMenu(self)
        newIcon = QIconSet(QPixmap(filenew))
        file.insertItem(newIcon,'&New Window',self.slotNewWindow,Qt.CTRL+Qt.Key_N)
        openIcon = QIconSet(QPixmap(fileopen))
        file.insertItem(openIcon,'&Open File',self.slotOpenFile,Qt.CTRL+Qt.Key_O)
        file.insertItem('&Print',self.slotPrint,Qt.CTRL+Qt.Key_P)
        file.insertSeparator()
        file.insertItem('&Close',self,SLOT('close()'),Qt.CTRL+Qt.Key_Q)
        file.insertItem('E&xit',qApp,SLOT('closeAllWindows()'),Qt.CTRL+Qt.Key_X)

        backIcon = QIconSet(QPixmap(back))
        forwardIcon = QIconSet(QPixmap(forward))
        homeIcon = QIconSet(QPixmap(homes))

        go = QPopupMenu(self)

        self.backwardId = go.insertItem(backIcon,'&Backward',self.browser,SLOT('backward()'),Qt.CTRL+Qt.Key_Left)
        self.forwardId = go.insertItem(forwardIcon,'&Forward',self.browser,SLOT('forward()'),Qt.CTRL+Qt.Key_Right)
        go.insertItem(homeIcon,'&Home',self.browser,SLOT('home()'))

        help = QPopupMenu(self)
        help.insertItem('&About',self.slotAbout)
        help.insertItem('About &Qt',self.slotAboutQt)

        self.hist = QPopupMenu(self)
        for it in self.history:
            self.mHistory[self.hist.insertItem(it)] = it
        self.connect(self.hist,SIGNAL('activated(int)'),self.slotHistChosen)

        self.bookm = QPopupMenu(self)
        self.bookm.insertItem('Add Bookmark',self.slotAddBookmark)
        self.bookm.insertSeparator()

        for it2 in self.bookmarks:
            self.mBookmarks[self.bookm.insertItem(it2)] = it2
        self.connect(self.bookm,SIGNAL('activated(int)'),self.slotBookmChosen)

        self.menuBar().insertItem('&File',file)
        self.menuBar().insertItem('&Go',go)
        self.menuBar().insertItem('History',self.hist)
        self.menuBar().insertItem('Bookmarks',self.bookm)
        self.menuBar().insertSeparator()
        self.menuBar().insertItem('&Help',help)

        self.menuBar().setItemEnabled(self.forwardId,FALSE)
        self.menuBar().setItemEnabled(self.backwardId,FALSE)

        self.connect(self.browser,SIGNAL('backwardAvailable(bool)'),self.setBackwardAvailable)
        self.connect(self.browser,SIGNAL('forwardAvailable(bool)'),self.setForwardAvailable)

        toolbar = QToolBar(self)

        button = QToolButton(backIcon,'Backward','',self.browser,SLOT('backward()'),toolbar)
        self.connect(self.browser,SIGNAL('backwardAvailable(bool)'),button,SLOT('setEnabled(bool)'))
        button.setEnabled(FALSE)
        button = QToolButton(forwardIcon,'Forward','',self.browser,SLOT('forward()'),toolbar)
        self.connect(self.browser,SIGNAL('forwardAvailable(bool)'),button,SLOT('setEnabled(bool)'))
        button.setEnabled(FALSE)
        button = QToolButton(homeIcon,'Home','',self.browser,SLOT('home()'),toolbar)

        toolbar.addSeparator()

        self.pathCombo = QComboBox(TRUE,toolbar)
        self.connect(self.pathCombo,SIGNAL('activated(const QString&)'),self.slotPathSelected)
        toolbar.setStretchableWidget(self.pathCombo)
        self.setRightJustification(TRUE)
        self.setDockEnabled(Qt.DockLeft,FALSE)
        self.setDockEnabled(Qt.DockRight,FALSE)

        self.pathCombo.insertItem(home_)
        self.browser.setFocus()

    def setBackwardAvailable(self,b):
        self.menuBar().setItemEnabled(self.backwardId,b)

    def setForwardAvailable(self,b):
        self.menuBar().setItemEnabled(self.forwardId,b)

    def slotSourceChanged(self,url):
        if self.browser.documentTitle().isNull():
            self.setCaption('PyQt - Example - Helpviewer' + url)
        else:
            self.setCaption('PyQt - Example - Helpviewer - ' + str(self.browser.documentTitle()))

        if not url.isEmpty()and self.pathCombo:
            exists = FALSE
            for i in range(self.pathCombo.count()):
                if str(self.pathCombo.text(i)) == str(url):
                    exists = TRUE
                    break
            if exists == FALSE:
                self.pathCombo.insertItem(url,0)
                self.pathCombo.setCurrentItem(0)
                self.mHistory[self.hist.insertItem(url)] = str(url)
            else:
                self.pathCombo.setCurrentItem(i)

    def __del__(self):
        history = self.mHistory
        f = QFile(str(QDir.currentDirPath()) + '/.history')
        f.open(IO_WriteOnly)
        for it in history.keys():
            f.writeBlock(history[it] + '\n')
        f.close()

        bookmarks = self.mBookmarks
        f2 = QFile(str(QDir.currentDirPath()) + '/.bookmarks')
        f2.open(IO_WriteOnly)
        for it in bookmarks.keys():
            f2.writeBlock(bookmarks[it] + '\n')
        f2.close()


    def slotAbout(self):
        QMessageBox.about(self,'HelpViewer Example','''
                        <p>This example implements a simple HTML help viewer
                        using PyQt's rich text capabilities</p>
                        <p>It's just about 100 lines of Python code, so don't expect too much :-)</p>''')

    def slotAboutQt(self):
        QMessageBox.aboutQt(self,'QBrowser')

    def slotOpenFile(self):
        fn = QFileDialog.getOpenFileName('','',self)
        if not fn.isEmpty():
            self.browser.setSource(fn)

    def slotNewWindow(self):
        self.w = HelpWindow(self.browser.source(),QStringList('qbrowser'))
        self.w.show()

    def slotPrint(self):
        printer = QPrinter()
        printer.setFullPage(TRUE)
        if printer.setup(self):
            p = QPainter(printer)
            metrics = QPaintDeviceMetrics(p.device())
            dpix = metrics.logicalDpiX()
            dpiy = metrics.logicalDpiY()
            margin = 72
            body = QRect(margin*dpix/72,margin*dpiy/72,metrics.width()-margin*dpix/72*2,metrics.height()-margin*dpiy/72*2)
            richText = QSimpleRichText(self.browser.text(),QFont(),self.browser.context(),self.browser.styleSheet(),self.browser.mimeSourceFactory(),body.height())
            richText.setWidth(p,body.width())
            view = QRect(body)
            page = 1
            while TRUE:
                richText.draw(p,body.left(),body.top(),view,self.colorGroup())
                view.moveBy(0,body.height())
                p.translate(0,-body.height())
                p.drawText(view.right()-p.fontMetrics().width(str(page)),view.bottom()+p.fontMetrics().ascent()+5,str(page))
                if view.top() >= richText.height():
                    break
                printer.newPage()
                page = page + 1
            p.end()

    def slotPathSelected(self,_path):
        _path = str(_path)
        self.browser.setSource(_path)
        for i in self.mHistory.keys():
            if self.mHistory[i] == _path:
                self.mHistory[self.hist.insertItem(_path)] = _path

    def readHistory(self):
        if QFile.exists(str(QDir.currentDirPath()) + '/.history'):
            f = QFile(str(QDir.currentDirPath()) + '/.history')
            f.open(IO_ReadOnly)
            while not f.atEnd():
                s = QString()
                f.readLine(s,10000)
                s = str(s)
                self.history.append(str(s[:-1]))
            f.close()

    def readBookmarks(self):
        if QFile.exists(str(QDir.currentDirPath()) + '/.bookmarks'):
            f = QFile(str(QDir.currentDirPath()) + '/.bookmarks')
            f.open(IO_ReadOnly)
            while not f.atEnd():
                s = QString()
                f.readLine(s,10000)
                s = str(s)
                self.bookmarks.append(str(s[:-1]))
            f.close()

    def slotHistChosen(self,i):
        for it in self.mHistory.keys():
            if str(it) == str(i):
                self.browser.setSource(self.mHistory[it])

    def slotBookmChosen(self,i):
        for it in self.mBookmarks.keys():
            if str(it) == str(i):
                self.browser.setSource(self.mBookmarks[it])

    def slotAddBookmark(self):
        self.mBookmarks[self.bookm.insertItem(self.caption())] = str(self.browser.context())

QApplication.setColorSpec(QApplication.ManyColor)
a = QApplication(sys.argv)
l = sys.argv

qtdirenv = environ['QTDIR']
if qtdirenv != None:
    home = qtdirenv + '/doc/html/index.html'

help = HelpWindow(home,QStringList('.'),None,'help viewer')
help.setCaption('PyQt - Example - Helpviewer')
if QApplication.desktop().width() > 400 and QApplication.desktop().height() > 500:
    help.show()
else:
    help.showMaximized()

QObject.connect(qApp,SIGNAL('lastWindowClosed()'),qApp,SLOT('quit()'))

a.exec_loop()