#!/usr/bin/env python

import sys
from qt import *

TRUE  = 1
FALSE = 0

class LifeWidget(QFrame):
    def __init__(self,s,parent=None,name=None):
    	QWidget.__init__(self,parent,name)
    	self.SCALE = s
    	self.maxi = self.maxj = 50
    	self.MAXSIZE = 50
    	self.MINSIZE = 10
    	self.BORDER = 5

    	self.cells = []
        for t in range(2):
            for i in range(self.MAXSIZE+2):
                for j in range(self.MAXSIZE+2):
                    self.cells.append(FALSE)

    	self.current = 0

    def pos2index(self,x):
        return (x - self.BORDER)/self.SCALE + 1

    def index2pos(self,i):
        return (i -1)*self.SCALE + self.BORDER

    def clear(self):
        self.current = 0
        for t in range(2):
            for i in range(self.MAXSIZE+2):
                for j in range(self.MAXSIZE+2):
                    k = 2*t+(self.MAXSIZE+2)*i + j
                    self.cells[k] = FALSE
        self.repaint()

    def resizeEvent(self,e):
        self.maxi = (e.size().width() - 2*self.BORDER)/self.SCALE
        self.maxj = (e.size().height() - 2*self.BORDER)/self.SCALE

    def setPoint(self,i,j):
        if i < 1 or i > self.maxi or j < 1 or j > self.maxj: return
        k = 2*self.current + (self.MAXSIZE+2)*i + j
        self.cells[k] = TRUE
        self.repaint(self.index2pos(i),self.index2pos(j),self.SCALE,self.SCALE,FALSE)

    def mouseHandle(self,pos):
        i = self.pos2index(pos.x())
        j = self.pos2index(pos.y())
        self.setPoint(i,j)

    def mouseMoveEvent(self,e):
        self.mouseHandle(e.pos())

    def mousePressEvent(self,e):
        if e.button() == QMouseEvent.LeftButton:
            self.mouseHandle(e.pos())

    def nextGeneration(self):
        for i in range(1,self.MAXSIZE+1):
            for j in range(1,self.MAXSIZE+1):
                k = self.MAXSIZE+1
                k1 = self.current*2
                k2 =(self.MAXSIZE+1)*i
                k3 = j
                t = self.cells[k1+k2-k+k3-1]+self.cells[k1+k2-k+k3]+self.cells[k1+k2-k+k3+1]+self.cells[k1+k2+k3-1]+self.cells[k1+k2+k3+1]+self.cells[k1+k2+k+k3-1]+self.cells[k1+k2+k+k3]+self.cells[k1+k2+k+k3+1]
                if self.current:
                    self.cells[k2+k3] = t==3 or t==2 and self.cells[2+k2+k3]
                    self.current = FALSE
                else:
                    self.cells[2+k2+k3] = t==3 or t==2 and self.cells[k2+k3]
                    self.current = TRUE
        self.repaint(FALSE)

    def paintEvent(self,e):
        starti = self.pos2index(e.rect().left())
        stopi = self.pos2index(e.rect().right())
        startj = self.pos2index(e.rect().top())
        stopj = self.pos2index(e.rect().bottom())

        if stopi > self.maxi:stopi = self.maxi
        if stopj > self.maxj:stopj = self.maxj

        paint = QPainter(self)
        for i in range(starti,stopi+1):
            for j in range(startj,stopj+1):
                k1 = 2*self.current + (stopi+1)*i + j
                k2 = 2*(not self.current) + (stopi+1)*i + j
                if self.cells[k1]:
                    qDrawShadePanel(paint,self.index2pos(i),self.index2pos(j),self.SCALE-1,self.SCALE-1,self.colorGroup())
                elif self.cells[k2]:
                    self.erase(self.index2pos(i),self.index2pos(j),self.SCALE-1,self.SCALE-1)
        self.drawFrame(paint)

class LifeTimer(QTimer):
    def __init__(self,parent=None,name=None):
        QTimer.__init__(self,parent,name)

class LifeDialog(QFrame):
    def __init__(self,s,parent=None,name=None):
        QWidget.__init__(self,parent,name)
        self.TOPBORDER = 70
        self.SIDEBORDER = 10

        self.qb = QPushButton('Quit!',self)
        self.cb = QComboBox(self,'comboBox')
        self.life = LifeWidget(s,self)
        self.life.move(self.TOPBORDER,self.SIDEBORDER)

        self.connect(self.qb,SIGNAL('clicked()'),qApp,SLOT('quit()'))
        self.qb.setGeometry(self.TOPBORDER,self.SIDEBORDER,self.qb.sizeHint().width(),25)
        self.timer = LifeTimer(self)

        self.connect(self.timer,SIGNAL('timeout()'),self.life.nextGeneration)


## main program
a = QApplication(sys.argv)

scale = 10

life = LifeDialog(scale)
life.setCaption('PyQT Example - Life')
a.setMainWidget(life)
life.show()

a.exec_loop()