#!/usr/bin/env python

import sys
from qt import *

TRUE  = 1
FALSE = 0

class LineEdits(QGroupBox):
  def __init__(self,parent=None,name=None):
    QGroupBox.__init__(self,0,Qt.Horizontal,'Line edits',parent,name)

    self.setMargin(10)
        
    box = QVBoxLayout(self.layout())

    row1 = QHBoxLayout(box)
    row1.setMargin(5)

    label = QLabel("Echo Mode",self)
    row1.addWidget(label)

    self.combo1 = QComboBox(FALSE,self)
    row1.addWidget(self.combo1)
    self.combo1.insertItem("Normal",-1)
    self.combo1.insertItem("Password",-1)
    self.combo1.insertItem("No Echo",-1)

    QObject.connect(self.combo1,SIGNAL('activated(int)'),self.slotEchoChanged)

    self.lined1 = QLineEdit(self)
    box.addWidget(self.lined1)

    row2 = QHBoxLayout(box)
    row2.setMargin(5)

    label = QLabel("Validator:",self)
    row2.addWidget(label)

    self.combo2 = QComboBox(FALSE,self)
    row2.addWidget(self.combo2)
    self.combo2.insertItem("No Validator",-1)
    self.combo2.insertItem("Integer Validator",-1)
    self.combo2.insertItem("Double Validator", -1)
    self.connect(self.combo2,SIGNAL('activated(int)'),self.slotValidatorChanged)

    self.lined2 = QLineEdit(self)
    box.addWidget(self.lined2)

    row3 = QHBoxLayout(box)
    row3.setMargin(5)

    label = QLabel("Alignment:",self)
    row3.addWidget(label)

    self.combo3 = QComboBox(FALSE,self)
    row3.addWidget(self.combo3)
    self.combo3.insertItem("Left",-1)
    self.combo3.insertItem("Centered",-1)
    self.combo3.insertItem("Right",-1)
    self.connect(self.combo3,SIGNAL('activated(int)'),self.slotAlignmentChanged)

    self.lined3 = QLineEdit(self)
    box.addWidget(self.lined3)

    row4 = QHBoxLayout(box)
    row4.setMargin(5)

    label = QLabel("Read-Only:",self)
    row4.addWidget(label)

    self.combo4 = QComboBox(FALSE,self)
    row4.addWidget(self.combo4)
    self.combo4.insertItem("False",-1)
    self.combo4.insertItem("True",-1)
    self.connect(self.combo4,SIGNAL('activated(int)'),self.slotReadOnlyChanged)

    self.lined4 = QLineEdit(self)
    box.addWidget(self.lined4)

    self.lined1.setFocus() 

#
#  slotEchoChanged(int)
# 
#  int contains the number of the item which the user has been chosen in the
#  first Combobox. According to this value, we set the Echo-Mode for the
#  first LineEdit.
#

  def slotEchoChanged(self,int):
    if int == 0:
      self.lined1.setEchoMode(QLineEdit.Normal)
    elif int == 1:
      self.lined1.setEchoMode(QLineEdit.Password)   
    elif int == 2:
      self.lined1.setEchoMode(QLineEdit.NoEcho)
    self.lined1.setFocus()

#
#  slotValidatorChanged(int)
# 
#  int contains the number of the item which the user has been chosen in the
#  second Combobox. According to this value, we set a validator for the
#  second LineEdit. A validator checks in a LineEdit each character which
#  the user enters and accepts it if it is valid, else the character gets
#  ignored and not inserted into the lineedit.
# 

  def slotValidatorChanged(self,int):
    if int == 0:
      self.lined2.setValidator(None)
    elif int == 1:
      v1 = QIntValidator(0,100,self)
      self.lined2.setValidator(v1)
    elif int == 2:
      self.validator=QDoubleValidator(-999.0, 999.0, 2, self.lined2)
      self.lined2.setValidator(self.validator)

    self.lined2.setText("")
    self.lined2.setFocus()

#
#  slotAlignmentChanged(int)
# 
#  int contains the number of the item which the user has been chosen in
#  the third Combobox.  According to this value, we set an alignment
#  third LineEdit.
# 

  def slotAlignmentChanged(self,int):
    if int == 0:
      self.lined3.setAlignment(QLineEdit.AlignLeft)
    elif int == 1:
      self.lined3.setAlignment(QLineEdit.AlignCenter)   
    elif int == 2:
      self.lined3.setAlignment(QLineEdit.AlignRight)
    self.lined3.setFocus()

#
#  slotReadOnlyChanged(int)
# 
#  int contains the number of the item which the user has been chosen in
#  the fourth Combobox.  According to this value, we toggle read-only.
# 

  def slotReadOnlyChanged(self,int):
    if int == 0:
      self.lined4.setReadOnly(FALSE)
    elif int == 1:
      self.lined4.setReadOnly(TRUE)   
    self.lined4.setFocus()

   
## main program
a = QApplication(sys.argv)

lineedits = LineEdits()
lineedits.resize(200,200)
lineedits.setCaption("PyQT Example - Line Edits")
a.setMainWidget(lineedits)
lineedits.show()

a.exec_loop()

