#!/usr/bin/env python

import sys, string
from qt import *

TRUE  = 1
FALSE = 0

class Folder(QObject):
    def __init__(self,parent=None,name=None):
        QObject.__init__(self,parent,name)
        self.fName = name
        self.lstMessages = []

    def addMessage(self,m):
        self.lstMessages.append(m)

    def folderName(self):
        return self.fName

    def messages(self):
        return self.lstMessages

class FolderListItem(QListViewItem):
    def __init__(self,parent,f):
        QListViewItem.__init__(self,parent)
        self.myFolder = f
        self.setText(0,f.folderName())

        if self.myFolder.children() != None:
            self.insertSubFolders(self.myFolder.children())

    def insertSubFolders(self,lst):
        for f in lst: FolderListItem(self,f)

    def folder(self):
        return self.myFolder

class Message:
    def __init__(self,mh,_body):
        self.mheader = mh
        self.mbody = _body
        self.Read = 0
        self.Unread = 1
        self.mstate = self.Unread

    def header(self):
        return self.mheader

    def setState(self,s):
        self.mstate = s

    def state(self):
        return self.mstate

    def body(self):
        return self.mbody

class MessageListItem(QListViewItem):
    def __init__(self,parent,m):
        QListViewItem.__init__(self,parent)
        self.myMessage = m
        self.setText(0,self.myMessage.header().sender())
        self.setText(1,self.myMessage.header().subject())
        self.setText(2,self.myMessage.header().datetime().toString())

    def message(self):
        return self.myMessage

    def paintCell(self,p,cg,column,_width,_alignment):
        _cg = QColorGroup(cg)
        c = QColor(_cg.text())

        if self.myMessage.state() == 1:
            _cg.setColor(QColorGroup.Text,Qt.red)

        QListViewItem.paintCell(self,p,_cg,column,_width,_alignment)

        _cg.setColor(QColorGroup.Text,c)

class MessageHeader:
    def __init__(self,_sender,_subject,_datetime):
        self.msender = _sender
        self.msubject = _subject
        self.mdatetime = _datetime

    def sender(self):
        return self.msender

    def subject(self):
        return self.msubject

    def datetime(self):
        return self.mdatetime

class ListViews(QSplitter):
    def __init__(self,parent=None,name=None):
        QSplitter.__init__(self,Qt.Horizontal,parent,name)
        self.lstFolders = []

        self.folders = QListView(self)
        self.folders.header().setClickEnabled(FALSE)
        self.folders.addColumn('Folder')

        self.initFolders()
        self.setupFolders()

        self.folders.setRootIsDecorated(TRUE)
        self.setResizeMode(self.folders,QSplitter.KeepSize)

        vsplitter = QSplitter(Qt.Vertical,self)

        self.messages = QListView(vsplitter)
        self.messages.addColumn('Sender')
        self.messages.addColumn('Subject')
        self.messages.addColumn('Date')
        self.messages.setColumnAlignment(1,Qt.AlignRight)
        self.messages.setAllColumnsShowFocus(TRUE)
        self.messages.setShowSortIndicator(TRUE)

        self.menu = QPopupMenu(self.messages)
        for i in range(10):
            self.menu.insertItem('Context Item ' + str(i))
        self.connect(self.messages,SIGNAL('contextMenuRequested(QListViewItem *,const QPoint&,int)'),self.slotRMB)
        vsplitter.setResizeMode(self.messages,QSplitter.PaletteBase)

        self.message = QLabel(vsplitter)
        self.message.setAlignment(Qt.AlignTop)
        self.message.setBackgroundMode(Qt.PaletteBase)

        self.connect(self.folders,SIGNAL('selectionChanged(QListViewItem*)'),self.slotFolderChanged)
        self.connect(self.messages,SIGNAL('selectionChanged()'),self.slotMessageChanged)
        self.connect(self.messages,SIGNAL('currentChanged(QListViewItem *)'),self.slotMessageChanged)

        self.messages.setSelectionMode(QListView.Extended)

        self.folders.firstChild().setOpen(TRUE)
        self.folders.firstChild().firstChild().setOpen(TRUE)
        self.folders.setCurrentItem(self.folders.firstChild().firstChild().firstChild())
        self.folders.setSelected(self.folders.firstChild().firstChild().firstChild(),TRUE)

        self.messages.setSelected(self.messages.firstChild(),TRUE)
        self.messages.setCurrentItem(self.messages.firstChild())
        self.message.setMargin(5)

        lst = []
        lst.append(170)
        self.setSizes(lst)

    def initFolders(self):
        mcount = 1
        for i in range(1,21):
            _str = 'Folder ' + str(i)
            f = Folder(None,_str)
            for j in range(1,6):
                str2 = 'Sub Folder ' + str(j)
                f2 = Folder(f,str2)
                for k in range(1,3):
                    str3 = 'Sub Sub Folder ' + str(k)
                    f3 = Folder(f2,str3)
                    self.initFolder(f3,mcount)
            self.lstFolders.append(f)

    def initFolder(self,folder,_count):
        for i in range(15):
            _str = 'Message ' + str(_count)
            dt = QDateTime.currentDateTime()
            dt = dt.addSecs(60 * _count)
            mh = MessageHeader('theKompany <support@thekomany.com>',_str,dt)

            body = 'This is the message number ' + str(_count) + ' of this application, \n'
            body = body + 'which shows how to use QListViews, QListViewItems, \n'
            body = body + 'QSplitters and so on. The code should show how easy\n'
            body = body +'this can be done in PyQt.'
            _count = _count + 1
            msg = Message(mh,body)
            folder.addMessage(msg)

    def setupFolders(self):
        self.folders.clear()
        for f in self.lstFolders:
            FolderListItem(self.folders,f)

    def slotRMB(self,Item,point):
        if Item !=None:
            self.menu.popup(point)

    def slotFolderChanged(self,i):
        if not i:
            return
        self.messages.clear()
        self.message.setText('')

        item = i

        for msg in item.folder().messages():
            MessageListItem(self.messages,msg)

    def slotMessageChanged(self):
        i = self.messages.currentItem()
        if not i:
            return

        item = i
        msg = item.message()
        tmp = msg.header().sender()
        tmp = tmp.replace('<','&lt;')
        tmp = tmp.replace('>','&gt;')
        text = QString('''<b><i>From:</i></b> <a href=\'mailto:info@trolltech.com\'>%1</a><br>
                        <b><i>Subject:</i></b> <big><big><b>%2</b></big></big><br>
                        <b><i>Date:</i></b>%3<br><br>
                        \n%4''').arg(tmp).arg(msg.header().subject()).arg(msg.header().datetime().toString()).arg(msg.body())

        self.message.setText(text)

        msg.setState(0)

## main program
a = QApplication( sys.argv )

listViews = ListViews()
listViews.resize(640,480)
listViews.setCaption('PyQt Example - ListViews')
a.setMainWidget(listViews)
listViews.show()

a.exec_loop()