#!/usr/bin/env python

import sys
from qt import *

TRUE  = 1
FALSE = 0

class ProgressBar(QButtonGroup):
    def __init__(self,parent=None,name=None):
	QButtonGroup.__init__(self,0,Qt.Horizontal,'Progress Bar',parent,name)

	self.setMargin(10)

	self.timer = QTimer()

	toplayout = QGridLayout(self.layout(),2,2,5)

	self.setRadioButtonExclusive(TRUE)

	self.slow = QRadioButton('S&low',self)
	self.normal = QRadioButton('&Normal',self)
	self.fast = QRadioButton('&Fast',self)

	vb1 = QVBoxLayout()
	toplayout.addLayout(vb1,0,0)

	vb1.addWidget(self.slow)
	vb1.addWidget(self.normal)
	vb1.addWidget(self.fast)

	self.start = QPushButton('&Start',self)
	reset = QPushButton('&Reset',self)

	vb2 = QVBoxLayout()
	toplayout.addLayout(vb2,0,1)

	vb2.addWidget(self.start)
	vb2.addWidget(reset)

	self.progress = QProgressBar(100,self)
	toplayout.addMultiCellWidget(self.progress,1,1,0,1)

	self.connect(self.start,SIGNAL('clicked()'),self.slotStart)
	self.connect(reset,SIGNAL('clicked()'),self.slotReset)

	self.connect(self.timer,SIGNAL('timeout()'),self.slotTimeout)

	self.normal.setChecked(TRUE)
	self.start.setFixedWidth(80)
	self.setMinimumWidth(300)

    def slotStart(self):
	if self.progress.progress() == -1:
	   if self.slow.isChecked():
	      self.progress.setTotalSteps(10000)
	   elif self.normal.isChecked():
	      self.progress.setTotalSteps(1000)
	   else:
	      self.progress.setTotalSteps(50)

	   self.slow.setEnabled(FALSE)
	   self.normal.setEnabled(FALSE)
	   self.fast.setEnabled(FALSE)

	if self.timer.isActive() == 0:
	   self.timer.start(1)
	   self.start.setText('&Pause')
	else:
	   self.timer.stop()
	   self.start.setText('&Continue')

    def slotReset(self):
	self.timer.stop()
	self.start.setText('&Start')
	self.start.setEnabled(TRUE)

	self.slow.setEnabled(TRUE)
	self.normal.setEnabled(TRUE)
	self.fast.setEnabled(TRUE)

	self.progress.reset()

    def slotTimeout(self):
	p = self.progress.progress()

	if p == self.progress.totalSteps():
	   self.start.setText('&Start')
	   self.start.setEnabled(FALSE)
	   return
	p = p + 1
	self.progress.setProgress(p)


## main program
a = QApplication(sys.argv)

progressbar = ProgressBar()
progressbar.setCaption('PyQt Example - ProgressBar')
a.setMainWidget(progressbar)
progressbar.show()

a.exec_loop()