#!/usr/bin/env python

import sys, string
from qt import *

TRUE  = 1
FALSE = 0

class FontRowTable(QFrame):
    def __init__(self,parent = None,name = None):
        QFrame.__init__(self,parent,name)
    	self.setBackgroundMode(QFrame.PaletteBase)
    	self.setFrameStyle(QFrame.Panel|QFrame.Sunken)
    	self.setMargin(8)
    	self.setRow(0)
    	print("test")
    	self.tablefont = QApplication.font()

    def sizeHint(self):
    	temp1 = QSize(16 * self.cellSize().width(),16 * self.cellSize().height())
    	temp2 = QSize((self.margin()+self.frameWidth()) * 2,(self.margin()+self.frameWidth()) * 2)
    	return QSize(temp1.width() + temp2.width(),temp1.height() + temp2.height())

    def cellSize(self):
    	fm = self.fontMetrics()
    	return QSize(fm.maxWidth(),fm.lineSpacing() + 1)

    def paintEvent(self,ev):
        QFrame.paintEvent(self,ev)
        p = QPainter(self)
        p.setClipRegion(ev.region())
    	r = ev.rect()
    	fm = QWidget.fontMetrics(self)
        if 0 > -fm.minLeftBearing():
            qmax = 0
        else:
            qmax =-fm.minLeftBearing()
        ml = self.frameWidth() + self.margin() + 1 + qmax
        mt = self.frameWidth() + self.margin()
        cell = QSize((self.width() - 15 - ml)/16,(self.height() - 15 - mt)/16)

        if not cell.width() or not cell.height():
            return

        mini = r.left()/cell.width()
        maxi = (r.right() + cell.width() - 1)/cell.width()
        minj = r.top()/cell.height()
        maxj = (r.bottom() + cell.height() - 1)/cell.height()

        h = fm.height()

        body = QColor(255,255,192)
        negative = QColor(255,192,192)
        positive = QColor(192,192,255)
        rnegative = QColor(255,128,128)
        rpositive = QColor(128,128,255)

        for j in range(minj,maxj):
            for i in range(mini,maxi):
                if i < 16 and j < 16:
                    x = i * cell.width()
                    y = j * cell.height()

                    ch = QChar(j*16+i+self.rows*256)

                    if fm.inFont(ch):
                        w = fm.width(ch)
                        l = fm.leftBearing(ch)
                        r = fm.rightBearing(ch)
    
                        x = x + ml
                        y = y + mt + h
    
                        p.fillRect(x,y,w,-h,QBrush(body))
    
                        if w:
                            if l:
                                if l>0:
                                    p.fillRect(x,y-h/2,abs(l),-h/2,QBrush(negative))
                                else:
                                    p.fillRect(x+l,y-h/2,abs(l),-h/2,QBrush(positive))
                            if r:
                                if r>0:
                                    p.fillRect(x+w-r,y+2,abs(r),-h/2,QBrush(rnegative))
                                else:
                                    p.fillRect(x+w,y+2,abs(l),-h/2,QBrush(rpositive))
    
                        s = QString(ch)
                        p.setPen(QPen(Qt.black))
                        p.drawText(x,y,s)

    def setRow(self,r):
    	self.rows = r
    	fm = self.fontMetrics()
    	string = 'mLB=' + str(fm.minLeftBearing()) + ' mRB=' + str(fm.minRightBearing()) + ' mW=' + str(fm.maxWidth())

    	self.emit(PYSIGNAL('fontInformation(string)'),(string,))
    	self.update()
    
    def chooseFont(self):
    	oldfont = self.tablefont
    	tf = QFontDialog.getFont(oldfont,self)
    	self.tablefont = tf[0]
    
    	if tf[1]:
    	   self.setFont(self.tablefont)
    	else:
    	   self.tablefont = oldfont

class FontDisplayer(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self,None)
	self.table = FontRowTable(self)
	self.status = QStatusBar(self)
	controls = QToolBar(self)
	QLabel('Row: ',controls)
        self.row = QSpinBox(0,255,1,controls)
	controls.addSeparator()
    	self.fontbutton = QPushButton('Font...',controls)
	
	self.connect(self.row,SIGNAL('valueChanged(int)'),self.table.setRow)
	self.connect(self.fontbutton,SIGNAL('clicked()'),self.table.chooseFont)
	self.connect(self.table,PYSIGNAL('fontInformation(string)'),self.status,SLOT('message(string)'))
    
    	self.table.setRow(0)
	self.setCentralWidget(self.table)
    
## main program
QApplication.setFont(QFont('unifont',16))
a = QApplication(sys.argv)

m = FontDisplayer()
sh = m.centralWidget().sizeHint()
m.resize(sh.width(),sh.height() + 3 * m.statusBar().height())
a.setMainWidget(m)
m.setCaption('PyQt Example - QFD')
m.show()

a.exec_loop()